#!/usr/bin/env python2
# http://oss.oetiker.ch/rrdtool/prog/rrdpython.en.html

# Release 0.01

###
### To Do:
###  - Define different colors in multiple device graphs
###  - Clean up labels
###
###  INFO: - partition uuid's are for keeping track of disk usage.
###        - drive_ids are disk id's for temperature tracking (/dev/disk/by-id)
###        - drive_activity is a python dictionary of user defined labels
###            followed by disk-id's for measuring drive activity
###            (/dev/disk/by-id)
###

import os
import sys
import rrdtool
from optparse import OptionParser
import ConfigParser
import subprocess
import re

DB_DIR="/var/lib/rrd"
PNG_DIR="/data/srv/httpd/htdocs/rrd"
cli_opts = []
drive_ids = [ '<replace_me_>',
              '<replace_me>',
              '<replace_me>',
              '<replace_me>' ]
partition_uuid = [ 'uuid-1',
                   'uuid-2',
                   'uuid-3',
                   'uuid-4',
                   'uuid-5' ]
drive_activity = { 'OS_Drive': '<my_disk-id>',
                   'MythTV_Drive': '<my_disk-id>' }
interface = 'eth0'
periods = [ 'day', 'week', 'month', 'year' ]

##
## Define our options
##

def commandline():

    global cli_opts

    parser = OptionParser()
    parser.add_option("-c","--cpu", action="store_true", default=False, help="Use RRDTool to measure CPU usage.")
    parser.add_option("-u","--usage", action="store_true", default=False, help="Use RRDTool to measure disk usage.")
    parser.add_option("-t","--hddtemp", action="store_true", default=False, help="Use RRDTool to measure hard drive temperature.")
    parser.add_option("-s","--system", action="store_true", default=False, help="Use RRDTool to measure system stats.")
    parser.add_option("-f","--fans", action="store_true", default=False, help="Use RRDTool to display fan speed.")
    parser.add_option("-n","--network", action="store_true", default=False, help="Use RRDTool to measure network usage.")
    parser.add_option("-a","--activity", action="store_true", default=False, help="Use RRDTool to measure disk activity.")
    parser.add_option("-m","--memory", action="store_true", default=False, help="Use RRDTool to measure memory usage.")
    parser.add_option("-p","--ups", action="store_true", default=False, help="Use RRDTool to display UPS statistics.")

    (options, args) = parser.parse_args()

    xyz = ['cpu', 'usage', 'hddtemp', 'system', 'fans', 'network', 'activity', 'memory', 'ups' ]
    for o in xyz:
        cmd = eval('options.'+o)
        if cmd is True:
            cli_opts.append('--'+o)

    return

##
## Database check/creation
##

def cpudb():

    DB = 'cpu'
    if not os.path.isfile( DB_DIR + "/" + DB + ".rrd" ):
        data_sources=[ 'DS:user:COUNTER:600:0:U',
                       'DS:system:COUNTER:600:0:U',
                       'DS:idle:COUNTER:600:0:U' ]
        rrdtool.create( DB_DIR + "/" + DB + ".rrd",
                        '-s 300',
                        data_sources,
                        'RRA:AVERAGE:0.5:1:576',
                        'RRA:AVERAGE:0.5:6:672',
                        'RRA:AVERAGE:0.5:24:732',
                        'RRA:AVERAGE:0.5:144:1460' )
    else:
        print DB, "database already present!"

    return

def usagedb():

    partition_map = {}
    data_sources = []
    disk_path = '/dev/disk/by-uuid'
    DB = 'usage'

    for u in partition_uuid:
        device = os.path.realpath(disk_path + "/" + u)
        p1 = subprocess.Popen(['/bin/mount', '-v'], stdout=subprocess.PIPE, stderr=subprocess.STDOUT,)
        for line in p1.stdout:
            data = re.match(device, line)
            if data:
                dir = re.split(r"\s+", line)
                mount_point = dir[2]
                name = os.path.basename(mount_point)
                if name == '':
                    name = 'rootfs'
                partition_map[ device ] = name
                data_sources.append( 'DS:' + name + '_used:GAUGE:600:0:U' )
                data_sources.append( 'DS:' + name + '_available:GAUGE:600:0:U' )

    if not os.path.isfile( DB_DIR + "/" + DB + ".rrd" ):
        rrdtool.create( DB_DIR + "/" + DB + ".rrd",
                        '-s 300',
                        data_sources,
                        'RRA:AVERAGE:0.5:1:432',
                        'RRA:AVERAGE:0.5:2:1008',
                        'RRA:AVERAGE:0.5:6:1440',
                        'RRA:AVERAGE:0.5:12:8760',
                        'RRA:MAX:0.5:6:336',
                        'RRA:MAX:0.5:12:720',
                        'RRA:MAX:0.5:24:4380' )
    else:
        print DB, "database already present!"

    return partition_map

def hddtempdb():

    disk_map = {}
    data_sources = []
    disk_path = '/dev/disk/by-id'
    DB = 'hddtemp'

    for i in drive_ids:
        device_path = os.path.realpath(disk_path + "/" + i)
        p1 = subprocess.Popen(['/sbin/hdparm', '-I', device_path ], stdout=subprocess.PIPE, stderr=subprocess.STDOUT,)
        p2 = subprocess.Popen(['/bin/grep', "Serial Number" ], stdin=p1.stdout, stdout=subprocess.PIPE, stderr=subprocess.STDOUT,)
        for line in p2.stdout:
            data = re.split(r"\s+", line)
            serial_number = data[3]
            disk_map[ device_path ] = serial_number
            data_sources.append( 'DS:' + serial_number + ':GAUGE:600:0:100' )

    if not os.path.isfile( DB_DIR + "/" + DB + ".rrd" ):
        rrdtool.create( DB_DIR + "/" + DB + ".rrd",
                        '-s 300',
                        data_sources,
                        'RRA:AVERAGE:0.5:1:576',
                        'RRA:AVERAGE:0.5:6:672',
                        'RRA:AVERAGE:0.5:24:732',
                        'RRA:AVERAGE:0.5:144:1460' )
    else:
        print DB, "database already present!"

    return disk_map

def systemdb():

    DB = 'system'
    if not os.path.isfile( DB_DIR + "/" + DB + ".rrd" ):
        data_sources=[ 'DS:cpu_temp:GAUGE:300:0:100',
                       'DS:mb_temp:GAUGE:300:0:100',
                       'DS:vcore:GAUGE:300:0:3',
                       'DS:avcc:GAUGE:300:2:4',
                       'DS:three_v:GAUGE:300:2:4',
                       'DS:three_vsb:GAUGE:300:2:4',
                       'DS:vbat:GAUGE:300:2:4' ]

        rrdtool.create( DB_DIR + "/" + DB + ".rrd",
                        '-s 300',
                        data_sources,
                        'RRA:AVERAGE:0.5:1:12',
                        'RRA:AVERAGE:0.5:1:288',
                        'RRA:AVERAGE:0.5:12:168',
                        'RRA:AVERAGE:0.5:12:720',
                        'RRA:AVERAGE:0.5:288:365' )
    else:
        print DB, "database already present!"

    return

def networkdb():

    DB = 'network'
    if not os.path.isfile( DB_DIR + "/" + DB + ".rrd" ):
        data_sources=[ 'DS:rx:COUNTER:600:0:U',
                       'DS:tx:COUNTER:600:0:U' ]
        rrdtool.create( DB_DIR + "/" + DB + ".rrd",
                        '-s 300',
                        data_sources,
                        'RRA:AVERAGE:0.5:1:576',
                        'RRA:AVERAGE:0.5:6:672',
                        'RRA:AVERAGE:0.5:24:732',
                        'RRA:AVERAGE:0.5:144:1460' )
    else:
        print DB, "database already present!"

    return

def activitydb():

    DB = 'activity'
    disk_path = '/dev/disk/by-id'
    data_sources = []

    # Replace the /dev/disk/by-id name with the current assignment (i.e. 'sda')
    for k in drive_activity.keys():
        v = drive_activity.get(k)
        device_path = os.path.realpath(disk_path + "/" + v)
        name = os.path.basename( device_path )
        drive_activity[ k ] = name

    for i in drive_activity.keys():
        data_sources.append( 'DS:' + i + '_read' + ':GAUGE:600:0:U' )
        data_sources.append( 'DS:' + i + '_write' + ':GAUGE:600:0:U' )

    if not os.path.isfile( DB_DIR + "/" + DB + ".rrd" ):
        rrdtool.create( DB_DIR + "/" + DB + ".rrd",
                        '-s 300',
                        data_sources,
                        'RRA:AVERAGE:0.5:1:576',
                        'RRA:AVERAGE:0.5:6:672',
                        'RRA:AVERAGE:0.5:24:732',
                        'RRA:AVERAGE:0.5:144:1460' )
    else:
        print DB, "database already present!"

    return drive_activity

def memorydb():

    DB = 'memory'
    if not os.path.isfile( DB_DIR + "/" + DB + ".rrd" ):
        data_sources=[ "DS:mem_used:ABSOLUTE:600:0:U",
                       "DS:mem_free:ABSOLUTE:600:0:U",
                       "DS:mem_shared:ABSOLUTE:600:0:U",
                       "DS:mem_buffers:ABSOLUTE:600:0:U",
                       "DS:mem_cache:ABSOLUTE:600:0:U",
                       "DS:swap_used:ABSOLUTE:600:0:U",
                       "DS:swap_free:ABSOLUTE:600:0:U" ]

        rrdtool.create( DB_DIR + "/" + DB + ".rrd",
                        '-s 300',
                        data_sources,
                        'RRA:AVERAGE:0.5:1:576',
                        'RRA:AVERAGE:0.5:6:672',
                        'RRA:AVERAGE:0.5:24:732',
                        'RRA:AVERAGE:0.5:144:1460' )
    else:
        print DB, "database already present!"

    return

def upsdb():

    DB = 'ups'
    if not os.path.isfile( DB_DIR + "/" + DB + ".rrd" ):
        data_sources=[ 'DS:load:GAUGE:600:0:U',
                       'DS:line:GAUGE:600:0:U',
                       'DS:charge:GAUGE:600:0:U',
                       'DS:time:GAUGE:600:0:U' ]
        rrdtool.create( DB_DIR + "/" + DB + ".rrd",
                        '-s 300',
                        data_sources,
                        'RRA:AVERAGE:0.5:1:576',
                        'RRA:AVERAGE:0.5:6:672',
                        'RRA:AVERAGE:0.5:24:732',
                        'RRA:AVERAGE:0.5:144:1460' )
    else:
        print DB, "database already present!"

    return

def fansdb():

    DB = 'fans'
    data_sources = []
    # Count the number of fans to monitor
    total_fans = 0

    p1 = subprocess.Popen(['/usr/bin/sensors'], stdout=subprocess.PIPE, stderr=subprocess.STDOUT,)
    for line in p1.stdout:
        search_fan = re.search('RPM', line)
        if search_fan:
            total_fans = total_fans + 1
    total = range(0, total_fans)
    if total:
        for n in total:
            data_sources.append( 'DS:fan' + str(n) + ':GAUGE:600:0:10000' )

    if data_sources:
        if not os.path.isfile( DB_DIR + "/" + DB + ".rrd" ):
            rrdtool.create( DB_DIR + "/" + DB + ".rrd",
                            '-s 300',
                            data_sources,
                            'RRA:AVERAGE:0.5:1:576',
                            'RRA:AVERAGE:0.5:6:672',
                            'RRA:AVERAGE:0.5:24:732',
                            'RRA:AVERAGE:0.5:144:1460' )
        else:
            print DB, "database already present!"
    else:
        print "No fans found!"

    return total_fans

##
## Update the database
##

def cpuup():

    DB = 'cpu'
    if os.path.isfile( DB_DIR + "/" + DB + ".rrd" ):
        f = open('/proc/stat', 'r')
        cpu_stat = f.readlines()
        f.close()

        # Iterate through the lines looking for a match pattern
        for line in cpu_stat:
            stat_line = re.match(r"cpu\s", line)
            # When a match is found, extract the data
            if stat_line:
                data = re.split(r"\s+", line)    
                # Add user and nice
                tmp_user = int(data[1]) + int(data[2])
                real_user = str(tmp_user)
                real_idle = str(data[4])
                real_system = str(data[3])

        rrdtool.update( DB_DIR + "/" + DB + ".rrd",
                        "-t",
                        'user:system:idle',
                        "N:" + real_user + ":" + real_system + ":" + real_idle )
    else:
        print DB_DIR + "/" + DB + ".rrd not found! Skipping update of", DB

    return

def usageup(partition_map):

    DB = 'usage'
    if os.path.isfile( DB_DIR + "/" + DB + ".rrd" ):
        data = {}
        p1 = subprocess.Popen(['/bin/df', '-P'], stdout=subprocess.PIPE, stderr=subprocess.STDOUT,)
        for line in p1.stdout:
            for n in partition_map.keys():
                if n in line:
                    diskline = re.split(r"\s+", line)
                    name = partition_map[n]
                    data[ name + "_used"] = str(diskline[2])
                    data[ name + "_available"] = str(diskline[3])

        # Format the dictionary keys and values for RRD
        drives = ":".join(data.keys())
        vals = ":".join(data.values())

        rrdtool.update( DB_DIR + "/" + DB + ".rrd",
                        "-t" + drives,
                        'N:' + vals )

    else:
        print DB_DIR + "/" + DB + ".rrd not found! Skipping update of", DB

    return

def hddtempup(disk_map):

    DB = 'hddtemp'
    val = {}
    for x in disk_map.keys():
        device = disk_map[x]
        # Using Popen instead of check_call for python2.6 compatability
        p1 = subprocess.Popen(['/usr/sbin/hddtemp', '-n', x], stdout=subprocess.PIPE, stderr=subprocess.STDOUT,)
        for line in p1.stdout:
            t = re.match(r"\d+", line)
            if t:
                temp = re.match(r"\d+", line).group().strip('\n')
                val[ device ] = temp

    if os.path.isfile( DB_DIR + "/" + DB + ".rrd" ):
        rrdtool.update( DB_DIR + "/" + DB + ".rrd",
                        "-t",
                        ":".join(val.keys()),
                        'N:' + ":".join(val.values()) )
    else:
        print DB_DIR + "/" + DB + ".rrd not found! Skipping update of", DB

    return

def systemup():

    core_temps = []
    voltages = {}
    DB = 'system'
    if os.path.isfile( DB_DIR + "/" + DB + ".rrd" ):
        p1 = subprocess.Popen(['/usr/bin/sensors'], stdout=subprocess.PIPE, stderr=subprocess.STDOUT,)
        for line in p1.stdout:
             match_cpu = re.match(r"Core\d Temp", line)
             match_mb = re.match(r"MB Temp", line)
             match_vcore = re.match(r"Vcore", line)
             match_avcc = re.match(r"AVCC", line)
             match_33v = re.match(r"\+3.3V", line)
             match_3vsb = re.match(r"3VSB", line)
             match_vbat = re.match(r"Vbat", line)
             if match_cpu:
                 core_cpu = re.search(r"\+\d+", line).group().lstrip('+')
                 core_temps.append(int(core_cpu))
                 if core_temps:
                     divisor = len(core_temps)
                     temp_cpu = sum(divmod(sum(core_temps), divisor), 21)
             if match_mb:
                 temp_mb = re.search(r"\d+", line).group()
             if match_vcore:
                 value = re.split(r"\s+", line)
                 voltages[ 'vcore' ] = value[1].lstrip('+')
             if match_avcc:
                 value = re.split(r"\s+", line)
                 voltages[ 'avcc' ] = value[1].lstrip('+')
             if match_33v:
                 value = re.split(r"\s+", line)
                 voltages[ 'three_v' ] = value[1].lstrip('+')
             if match_3vsb:
                 value = re.split(r"\s+", line)
                 voltages[ 'three_vsb' ] = value[1].lstrip('+')
             if match_vbat:
                 value = re.split(r"\s+", line)
                 voltages[ 'vbat' ] = value[1].lstrip('+')

        rrdtool.update( DB_DIR + "/" + DB + ".rrd",
                        "-t",
                        'cpu_temp:mb_temp:' + ":".join(voltages.keys()),
                        "N:" + str(temp_cpu) + ":" + temp_mb + ":" + ":".join(voltages.values()))

    else:
        print DB_DIR + "/" + DB + ".rrd not found! Skipping update of", DB

    return

def fansup(total_fans):

    DB = 'fans'
    if os.path.isfile( DB_DIR + "/" + DB + ".rrd" ):
        total = range(0, total_fans)
        data_sources = {}
        data_values = []
        val = {}

        if total:
            p1 = subprocess.Popen(['/usr/bin/sensors'], stdout=subprocess.PIPE, stderr=subprocess.STDOUT,)
            for line in p1.stdout:
                search_fan = re.search('RPM', line)
                if search_fan:
                        fan = 'fan' + str(total.pop())
                        speed = re.search(r"\d\d+", line).group()
                        val[ fan ] = speed

            rrdtool.update( DB_DIR + "/" + DB + ".rrd",
                            "-t",
                            ":".join(val.keys()),
                            'N:' + ":".join(val.values()) )
    else:
        print DB_DIR + "/" + DB + ".rrd not found! Skipping update of", DB

    return

def networkup():

    DB = 'network'
    if os.path.isfile( DB_DIR + "/" + DB + ".rrd" ):
        p1=subprocess.Popen(['/sbin/ifconfig', interface], stdout=subprocess.PIPE, stderr=subprocess.STDOUT,)
        for line in p1.stdout:
            # Commented lines are old nettools format
            #rx_match = re.search("RX bytes:\d+", line)
            #tx_match = re.search("TX bytes:\d+", line)
            rx_match = re.search(r"RX packets\s+\d+\s+bytes\s+\d+", line)
            tx_match = re.search(r"TX packets\s+\d+\s+bytes\s+\d+", line)
            if rx_match:
                #rx_bytes = rx_match.group()
                rx_bytes = re.split(r"\s+", line)
                #input = rx_bytes.lstrip("TX bytes:")
                input = rx_bytes[5]
            if tx_match:
                #tx_bytes = rx_match.group()
                tx_bytes = re.split(r"\s+", line)
                #output = tx_bytes.lstrip("TX bytes:")
                output = tx_bytes[5]

        rrdtool.update( DB_DIR + "/" + DB + ".rrd",
                        "-t",
                        "rx:tx",
                        "N:" + input + ":" + output )
    else:
        print DB_DIR + "/" + DB + ".rrd not found! Skipping update of", DB

    return

def activityup(drive_activity):

    DB = 'activity'
    data = {}

    # Build a dictionary with the current read/write values for our desired
    # devices in kB/s.
    p1=subprocess.Popen(['/usr/bin/iostat', '-d' ], stdout=subprocess.PIPE, stderr=subprocess.PIPE,)
    for line in p1.stdout:
        for k in drive_activity.keys():
            v = drive_activity.get(k)
            if v in line:
                diskline = re.split(r"\s+", line)
                read_ = re.match(r"\d+\.\d+", diskline[2]).group()
                write_ = re.match(r"\d+\.\d+", diskline[3]).group()
                data[ k + "_read" ] = read_
                data[ k + "_write" ] = write_
        devices = ":".join(data.keys())
        vals = ":".join(data.values())

    if os.path.isfile( DB_DIR + "/" + DB + ".rrd" ):
        rrdtool.update( DB_DIR + "/" + DB + ".rrd",
                        "-t", devices,
                        'N:' + vals )
    else:
        print DB_DIR + "/" + DB + ".rrd not found! Skipping update of", DB

    return

def memoryup():

    DB = 'memory'
    if os.path.isfile( DB_DIR + "/" + DB + ".rrd" ):
        data = {}

        # Build a dictionary with the current usage values
        p1=subprocess.Popen(['/usr/bin/free', '-b', '-o' ], stdout=subprocess.PIPE, stderr=subprocess.STDOUT,)
        for line in p1.stdout:
            mem = re.match('Mem:',line)
            swp = re.match('Swap:',line)
            if mem:
                mem_val = re.split(r"\s+", line)
                data[ 'mem_used' ] = mem_val[2]
                data[ 'mem_free' ] = mem_val[3]
                data[ 'mem_shared' ] = mem_val[4]
                data[ 'mem_buffers' ] = mem_val[5]
                data[ 'mem_cache' ] = mem_val[6]
            if swp:
                swp_val = re.split(r"\s+", line)
                data[ 'swap_used' ] = swp_val[2]
                data[ 'swap_free' ] = swp_val[3]

        rrdtool.update( DB_DIR + "/" + DB + ".rrd",
                        "-t",
                        ":".join(data.keys()),
                        'N:' + ":".join(data.values()) )
    else:
        print DB_DIR + "/" + DB + ".rrd not found! Skipping update of", DB

    return

def upsup():

    global model
    global status

    DB = 'ups'
    if os.path.isfile( DB_DIR + "/" + DB + ".rrd" ):
        if os.path.isfile( '/var/log/apcupsd.status' ):
            f = open('/var/log/apcupsd.status', 'r')
            ups_stats = f.readlines()
            f.close()
            # Iterate through the lines looking for a match pattern
            for line in ups_stats:
                lvolt = re.match(r"LINEV", line)
                upsload = re.match(r"LOADPCT", line)
                batt = re.match(r"BCHARGE", line)
                timeleft = re.match(r"TIMELEFT", line)
                name = re.match(r"MODEL", line)
                current = re.match(r"STATUS", line)
                # When a match is found, extract the data
                if lvolt:
                    data = re.search("\d+", line)
                    linev = data.group()
                if upsload:
                    data = re.search("\d+", line)
                    loadpct = data.group()
                if batt:
                    data = re.search("\d+", line)
                    bcharge = data.group()
                if timeleft:
                    data = re.search("\d+", line)
                    time = data.group()
                # Name and model are for the graph only
                if name:
                    data = re.search(r"[^MODEL    : ].*$",line)
                    model = data.group()
                if current:
                    data = re.search(r"[^STATUS   : ].*$",line)
                    status = data.group()

            rrdtool.update( DB_DIR + "/" + DB + ".rrd",
                            "-t",
                            'load:line:charge:time',
                            "N:" + loadpct + ":" + linev + ":" + bcharge + ":" + time )
        else:
            print "/var/log/apcupsd.status not found! Unable to update", DB
    else:
        print DB_DIR + "/" + DB + ".rrd not found! Skipping update of", DB

    return

##
## Create the graphs
##

def cpugraph():

    DB = 'cpu'
    if os.path.isfile( DB_DIR + "/" + DB + ".rrd" ):
        for p in periods:
            rrdtool.graph( PNG_DIR + "/" + DB + "-" + p + ".png",
                           "--start", "-1" + p, "-aPNG", "-i", "-z",
                           "--alt-y-grid", "-w 600", "-h 100", "-l 0", "-u 100", "-r",
                           "--color", "SHADEA#EAE9EE",
                           "--color", "SHADEB#EAE9EE",
                           "--color", "BACK#EAE9EE",
                           "-t CPU usage per " + p,
                           "DEF:user=" + DB_DIR + "/" + DB + ".rrd:user:AVERAGE",
                           "DEF:system=" + DB_DIR + "/" + DB + ".rrd:system:AVERAGE",
                           "DEF:idle=" + DB_DIR + "/" + DB + ".rrd:idle:AVERAGE",
                           "CDEF:total=user,system,idle,+,+",
                           "CDEF:userpct=100,user,total,/,*",
                           "CDEF:systempct=100,system,total,/,*",
                           "CDEF:idlepct=100,idle,total,/,*",
                           "AREA:userpct#0000FF:user\\:",
                           "GPRINT:userpct:MAX:maximal user cpu\\:%3.2lf%%",
                           "GPRINT:userpct:AVERAGE:average user cpu\\:%3.2lf%%",
                           "GPRINT:userpct:LAST:current user cpu\\:%3.2lf%%\\j",
                           "STACK:systempct#FF0000:system\\:",
                           "GPRINT:systempct:MAX:maximal system cpu\\:%3.2lf%%",
                           "GPRINT:systempct:AVERAGE:average system cpu\\:%3.2lf%%",
                           "GPRINT:systempct:LAST:current system cpu\\:%3.2lf%%\\j",
                           "STACK:idlepct#00FF00:idle\\:",
                           "GPRINT:idlepct:MAX:maximal idle cpu\\:%3.2lf%%",
                           "GPRINT:idlepct:AVERAGE:average idle cpu\\:%3.2lf%%",
                           "GPRINT:idlepct:LAST:current idle cpu\\:%3.2lf%%\\j")
    else:
        print DB_DIR + "/" + DB + ".rrd not found! Skipping graph for", DB

    return

def usagegraph(partition_map):

    DB = 'usage'
    if os.path.isfile( DB_DIR + "/" + DB + ".rrd" ):
        for x in partition_map.keys():
            n = partition_map[x]
            for p in periods:
                if p == 'day':
                    shifter = str(86400)
                    shifterL = '1 Day     '
                elif p == 'week':
                    shifter = str(604800)
                    shifterL = '7 Day     '
                elif p == 'month':
                    shifter = str(1209600)
                    shifterL = '14 Day    '
                elif p == 'year':
                    shifter = str(4838400)
                    shifterL = '8 Week    '

                rrdtool.graph( PNG_DIR + "/" + DB + "-" + n + "-" + p + ".png",
                               "--start", "-1" + p, "-aPNG", "-i", "-z", "-r",
                               "--alt-y-grid", "-w 600", "-h 100", "-l 0",
                               "--color", "SHADEA#EAE9EE",
                               "--color", "SHADEB#EAE9EE",
                               "--color", "BACK#EAE9EE",
                               "--vertical-label", "Partition Size",
                               "-t", 'Disk Usage - ' + x + ' (' + n + ') - ' + p,
                               'DEF:' + n + '_used=' + DB_DIR + '/' + DB + '.rrd:' + n + '_used:AVERAGE',
                               'DEF:' + n + '_available=' + DB_DIR + '/' + DB + '.rrd:' + n + '_available:AVERAGE',
                               'DEF:' + n + '_oldused=' + DB_DIR + '/' + DB + '.rrd:' + n + '_used:AVERAGE:start=-1' + p + '-' + shifter,
                               'SHIFT:' + n + '_oldused:' + shifter,
                               'CDEF:' + n + '_Bused=' + n + '_used,1024,*',
                               'CDEF:' + n + '_Bavail=' + n + '_available,1024,*',
                               'CDEF:' + n + '_total=' + n + '_Bused,' + n + '_Bavail,+',
                               'CDEF:' + n + '_wused=' + n + '_oldused,1024,*',
                               'AREA:' + n + '_total#32CD32:Free Space',
                               'GPRINT:' + n + '_Bavail:MAX:Max\\: %3.2lf %sB',
                               'GPRINT:' + n + '_Bavail:AVERAGE:Avg\\: %3.2lf %sB',
                               'GPRINT:' + n + '_Bavail:LAST:Current\\: %3.2lf %sB\\j',
                               'AREA:' + n + '_Bused#4169E1:Used      ',
                               'LINE1:' + n + '_Bused#0033CC',
                               'GPRINT:' + n + '_Bused:MAX:Max\\: %3.2lf %sB',
                               'GPRINT:' + n + '_Bused:AVERAGE:Avg\\: %3.2lf %sB',
                               'GPRINT:' + n + '_Bused:LAST:Current\\: %3.2lf %sB\\j',
                               'LINE1:' + n + '_wused#FF33CC:' + shifterL,
                               'GPRINT:' + n + '_wused:MAX:Max\\: %3.2lf %sB',
                               'GPRINT:' + n + '_wused:AVERAGE:Avg\\: %3.2lf %sB',
                               'GPRINT:' + n + '_wused:LAST:Current\\: %3.2lf %sB\\j' )

    else:
        print DB_DIR + "/" + DB + ".rrd not found! Skipping graph for", DB

    return

def hddtempgraph(disk_map):

    DB = 'hddtemp'
    if os.path.isfile( DB_DIR + "/" + DB + ".rrd" ):
        data_sources1 = []
        data_sources2 = []
        for x in disk_map.keys():
            n = disk_map[x]
            data_sources1.append( 'DEF:' + n + '=' + DB_DIR + '/' + DB + '.rrd:' + n + ':AVERAGE', )

            data_sources2.append( 'LINE2:' + n + '#0000FF:' + x )
            data_sources2.append( 'GPRINT:' + n + ':MIN:  Min\\: %2.lf,' )
            data_sources2.append( 'GPRINT:' + n + ':MAX: Max\\: %2.lf,' )
            data_sources2.append( 'GPRINT:' + n + ':AVERAGE: Avg\\: %4.1lf,' )
            data_sources2.append( 'GPRINT:' + n + ':LAST: Current\\: %2.lf degrees C\\n' )

        for p in periods:
            rrdtool.graph( PNG_DIR + "/" + DB + "-" + p + ".png",
                           "--lazy",
                           "-s -1" + p,
                           "--color", "SHADEA#EAE9EE",
                           "--color", "SHADEB#EAE9EE",
                           "--color", "BACK#EAE9EE",
                           "-t", 'Hard Drive Temperature - ' + p,
                           "-h", "100", "-w", "600",
                           "-a", "PNG",
                           "-v degrees C",
                           "--slope-mode",
                           data_sources1,
                           data_sources2 )

    else:
        print DB_DIR + "/" + DB + ".rrd not found! Skipping graph for", DB

    return

def systemgraph():

    DB = 'system'
    if os.path.isfile( DB_DIR + "/" + DB + ".rrd" ):
        for p in periods:
            rrdtool.graph( PNG_DIR + "/" + DB + "-" + p + ".png",
                           "--lazy",
                           "-s -1" + p,
                           "--color", "SHADEA#EAE9EE",
                           "--color", "SHADEB#EAE9EE",
                           "--color", "BACK#EAE9EE",
                           "-t", 'System Temperature - ' + p,
                           "-h", "100", "-w", "600",
                           "-a", "PNG",
                           "-v degrees C",
                           "DEF:cpu_temp=" + DB_DIR + "/" + DB + ".rrd:cpu_temp:AVERAGE",
                           "DEF:mb_temp=" + DB_DIR + "/" + DB + ".rrd:mb_temp:AVERAGE",
                           "LINE2:cpu_temp#0000FF:CPU Temperature",
                           "GPRINT:cpu_temp:MIN:  Min\\: %2.lf",
                           "GPRINT:cpu_temp:MAX: Max\\: %2.lf",
                           "GPRINT:cpu_temp:AVERAGE: Avg\\: %4.1lf",
                           "GPRINT:cpu_temp:LAST: Current\\: %2.lf degrees C\\n",
                           "LINE2:mb_temp#00CD00:MB Temperature ",
                           "GPRINT:mb_temp:MIN:  Min\\: %2.lf",
                           "GPRINT:mb_temp:MAX: Max\\: %2.lf",
                           "GPRINT:mb_temp:AVERAGE: Avg\\: %4.1lf",
                           "GPRINT:mb_temp:LAST: Current\\: %2.lf degrees C\\n" )
            rrdtool.graph( PNG_DIR + "/voltages" + "-" + p + ".png",
                           "--lazy",
                           "-s -1" + p,
                           "--color", "SHADEA#EAE9EE",
                           "--color", "SHADEB#EAE9EE",
                           "--color", "BACK#EAE9EE",
                           "-t", 'System Voltages - ' + p,
                           "-h", "100", "-w", "600",
                           "-a", "PNG",
                           "-v Volts",
                           "DEF:vcore=" + DB_DIR + "/" + DB + ".rrd:vcore:AVERAGE",
                           "DEF:avcc=" + DB_DIR + "/" + DB + ".rrd:avcc:AVERAGE",
                           "DEF:three_v=" + DB_DIR + "/" + DB + ".rrd:three_v:AVERAGE",
                           "DEF:three_vsb=" + DB_DIR + "/" + DB + ".rrd:three_vsb:AVERAGE",
                           "DEF:vbat=" + DB_DIR + "/" + DB + ".rrd:vbat:AVERAGE",
                           "LINE1:vcore#0000FF:Vcore",
                           "GPRINT:vcore:MIN:  Min\\: %1.2lf",
                           "GPRINT:vcore:MAX: Max\\: %1.2lf",
                           "GPRINT:vcore:AVERAGE: Avg\\: %1.2lf",
                           "GPRINT:vcore:LAST: Current\\: %1.2lf V\\j",
                           "LINE1:avcc#00CD00:AVCC",
                           "GPRINT:avcc:MIN:  Min\\: %1.2lf",
                           "GPRINT:avcc:MAX: Max\\: %1.2lf",
                           "GPRINT:avcc:AVERAGE: Avg\\: %1.2lf",
                           "GPRINT:avcc:LAST: Current\\: %1.2lf V\\j",
                           "LINE1:three_v#FF0000:3.3V",
                           "GPRINT:three_v:MIN:  Min\\: %1.2lf",
                           "GPRINT:three_v:MAX: Max\\: %1.2lf",
                           "GPRINT:three_v:AVERAGE: Avg\\: %1.2lf",
                           "GPRINT:three_v:LAST: Current\\: %1.2lf V\\j",
                           "LINE1:three_vsb#00FF00:3VSB",
                           "GPRINT:three_vsb:MIN:  Min\\: %1.2lf",
                           "GPRINT:three_vsb:MAX: Max\\: %1.2lf",
                           "GPRINT:three_vsb:AVERAGE: Avg\\: %1.2lf",
                           "GPRINT:three_vsb:LAST: Current\\: %1.2lf V\\j",
                           "LINE1:vbat#00DC00:Vbat",
                           "GPRINT:vbat:MIN:  Min\\: %1.2lf",
                           "GPRINT:vbat:MAX: Max\\: %1.2lf",
                           "GPRINT:vbat:AVERAGE: Avg\\: %1.2lf",
                           "GPRINT:vbat:LAST: Current\\: %1.2lf V\\j" )
    else:
        print DB_DIR + "/" + DB + ".rrd not found! Skipping graph for", DB

    return

def fansgraph(total_fans):

    DB = 'fans'
    if os.path.isfile( DB_DIR + "/" + DB + ".rrd" ):
        total = range(0, total_fans)
        data_sources1 = []
        data_sources2 = []
        if total:
            for n in total:
                data_sources1.append( 'DEF:fan' + str(n) + '=' + DB_DIR + '/' + DB + '.rrd:fan' + str(n) + ':AVERAGE' )
                data_sources2.append( 'LINE2:fan' + str(n) + '#0000FF:Fan' + str(n) )
                data_sources2.append( 'GPRINT:fan' + str(n) + ':MIN:  Min\\: %4.lf' )
                data_sources2.append( 'GPRINT:fan' + str(n) + ':MAX: Max\\: %4.lf')
                data_sources2.append( 'GPRINT:fan' + str(n) + ':AVERAGE: Avg\\: %4.1lf' )
                data_sources2.append( 'GPRINT:fan' + str(n) + ':LAST: Current\\: %4.lf RPM\\n' )
            if data_sources1:
                if data_sources2:
                    for p in periods:
                        rrdtool.graph( PNG_DIR + "/" + DB + "-" + p + ".png",
                                       "--lazy",
                                       "-s -1" + p,
                                       "--color", "SHADEA#EAE9EE",
                                       "--color", "SHADEB#EAE9EE",
                                       "--color", "BACK#EAE9EE",
                                       "-t", 'Fan Speeds - ' + p,
                                       "-h", "100", "-w", "600",
                                       "-a", "PNG",
                                       "-v", "RPM",
                                       data_sources1,
                                       data_sources2 )
                else:
                    print "Error constructing graph definitions! Skipping graph of", DB
            else:
                print "Error constructing graph definitions! Skipping graph of", DB
        else:
            print "No fans found! Skipping graph of", DB
    else:
        print DB_DIR + "/" + DB + ".rrd not found! Skipping graph for", DB

    return

def networkgraph():

    DB = 'network'
    if os.path.isfile( DB_DIR + "/" + DB + ".rrd" ):
        for p in periods:
            rrdtool.graph( PNG_DIR + '/' + DB + '-' + p + '.png',
                           '-s -1' + p,
                           "-t", 'Traffic on ' + interface + ' - ' + p,
                           "--lazy",
                           "-h", "100", "-w", "600",
                           "-l 0",
                           "-a", "PNG",
                           "-v Mbits/sec",
                           'DEF:rx=' + DB_DIR + '/' + DB + '.rrd:rx:AVERAGE',
                           'DEF:tx=' + DB_DIR + '/' + DB + '.rrd:tx:AVERAGE',
                           "CDEF:rx_bits=rx,8,*,1048576,/",
                           "CDEF:tx_bits=tx,-1,*,8,*,1048576,/",
                           "CDEF:pos_tx_bits=tx,8,*,1048576,/",
                           "AREA:rx_bits#32CD32:Incoming",
                           "LINE1:rx_bits#336600",
                           "GPRINT:rx_bits:MAX: Max\\: %4.1lf Mbps",
                           "GPRINT:rx_bits:AVERAGE: Avg\\: %4.1lf Mbps",
                           "GPRINT:rx_bits:LAST: Current\\: %4.1lf Mbps\\j",
                           "AREA:tx_bits#4169E1:Outgoing",
                           "LINE1:tx_bits#0033CC",
                           "GPRINT:pos_tx_bits:MAX: Max\\: %4.1lf Mbps",
                           "GPRINT:pos_tx_bits:AVERAGE: Avg\\: %4.1lf Mbps",
                           "GPRINT:pos_tx_bits:LAST: Current\\: %4.1lf Mbps\\j" )

    else:
        print DB_DIR + "/" + DB + ".rrd not found! Skipping graph for", DB

    return

def activitygraph(drive_activity):

    DB = 'activity'

    if os.path.isfile( DB_DIR + "/" + DB + ".rrd" ):
        for p in periods:
            for k in drive_activity.keys():
                v = drive_activity.get(k)
                data_sources = []
                data_sources.append( 'DEF:' + k + '_read=' + DB_DIR + '/' + DB + '.rrd:' + k + '_read:AVERAGE' )
                data_sources.append( 'DEF:' + k + '_write=' + DB_DIR + '/' + DB + '.rrd:' + k + '_write:AVERAGE' )
                data_sources.append( 'CDEF:' + k + '_read_bytes=' + k + '_read,1024,*' )
                data_sources.append( 'CDEF:' + k + '_write_bytes=' + k + '_write,1024,*' )
                data_sources.append( 'AREA:' + k + '_read_bytes#32CD32:Reads per sec ' )
                data_sources.append( 'LINE1:' + k + '_read_bytes#336600' )
                data_sources.append( 'GPRINT:' + k + '_read_bytes:MAX:Max\\: %4.2lf %sB' )
                data_sources.append( 'GPRINT:' + k + '_read_bytes:AVERAGE:Average\\: %4.2lf %sB' )
                data_sources.append( 'GPRINT:' + k + '_read_bytes:LAST:Current\\: %4.2lf %sB\\j' )
                data_sources.append( 'AREA:' + k + '_write_bytes#4169E1:Writes per sec' )
                data_sources.append( 'LINE1:' + k + '_write_bytes#0033CC' )
                data_sources.append( 'GPRINT:' + k + '_write_bytes:MAX:Max\\: %4.2lf %sB' )
                data_sources.append( 'GPRINT:' + k + '_write_bytes:AVERAGE:Average\\: %4.2lf %sB' )
                data_sources.append( 'GPRINT:' + k + '_write_bytes:LAST:Current\\: %4.2lf %sB\\j' )

                rrdtool.graph( PNG_DIR + '/' + DB + '-' + k + '-' + p + '.png',
                               "--start", "-1" + p, "-aPNG", "-i", "-z",
                               "-w 600", "-h 100", "-l 0",
                               "--color", "SHADEA#EAE9EE",
                               "--color", "SHADEB#EAE9EE",
                               "--color", "BACK#EAE9EE",
                               "-t", 'Disk activity /dev/' + v + ' - ' + p,
                               "-v Activity in bytes",
                               data_sources )
    else:
        print DB_DIR + "/" + DB + ".rrd not found! Skipping graph for", DB
        #        data_sources.append( 'CDEF:' + k + '_neg_write_bytes=' + k + '_write_bytes,-1,*' )

    return

def memorygraph():

    DB = 'memory'
    if os.path.isfile( DB_DIR + "/" + DB + ".rrd" ):
        for p in periods:
            rrdtool.graph( PNG_DIR + '/' + DB + '-' + p + '.png',
                           "--start", "-1" + p, "-aPNG", "-i", "-z",
                           "--alt-y-grid", "-w 600", "-h 100", "-l 0", "-u 100", "-r",
                           "--color", "SHADEA#EAE9EE",
                           "--color", "SHADEB#EAE9EE",
                           "--color", "BACK#EAE9EE",
                           "-t", 'Memory usage' + ' - ' + p,
                           'DEF:used=' + DB_DIR + '/' + DB + '.rrd:mem_used:AVERAGE',
                           'DEF:free=' + DB_DIR + '/' + DB + '.rrd:mem_free:AVERAGE',
                           'DEF:shared=' + DB_DIR + '/' + DB + '.rrd:mem_shared:AVERAGE',
                           'DEF:buffer=' + DB_DIR + '/' + DB + '.rrd:mem_buffers:AVERAGE',
                           'DEF:cache=' + DB_DIR + '/' + DB + '.rrd:mem_cache:AVERAGE',
                           "CDEF:total=used,free,+",
                           "CDEF:used2=used,buffer,cache,shared,+,+,-",
                           "CDEF:usedpct=100,used2,total,/,*",
                           "CDEF:sharedpct=100,shared,total,/,*",
                           "CDEF:bufferpct=100,buffer,total,/,*",
                           "CDEF:cachepct=100,cache,total,/,*",
                           "CDEF:freepct=100,free,total,/,*",
                           "AREA:usedpct#0000FF:used memory\\:",
                           "GPRINT:usedpct:MAX: Max used\\:     %3.2lf%%",
                           "GPRINT:usedpct:AVERAGE:Ave used\\: %3.2lf%%",
                           "GPRINT:usedpct:LAST:Current used\\:  %3.2lf%%\\j",
                           "STACK:sharedpct#FF0000:shared memory\\:",
                           "GPRINT:sharedpct:MAX: Max shared\\:    %3.2lf%%",
                           "GPRINT:sharedpct:AVERAGE: Ave shared\\:  %3.2lf%%",
                           "GPRINT:sharedpct:LAST:Current shared\\:  %3.2lf%%\\j",
                           "STACK:bufferpct#FF00FF:buffered memory\\:",
                           "GPRINT:bufferpct:MAX:Max buffered\\: %3.2lf%%",
                           "GPRINT:bufferpct:AVERAGE:Ave buffered\\: %3.2lf%%",
                           "GPRINT:bufferpct:LAST:Current buffered\\:  %3.2lf%%\\j",
                           "STACK:cachepct#FFFF00:cached memory\\:",
                           "GPRINT:cachepct:MAX:  Max cached\\:   %3.2lf%%",
                           "GPRINT:cachepct:AVERAGE:Ave cached\\:   %3.2lf%%",
                           "GPRINT:cachepct:LAST:Current cached\\:  %3.2lf%%\\j",
                           "STACK:freepct#00FF00:free memory\\:",
                           "GPRINT:freepct:MAX: Max free\\:     %3.2lf%%",
                           "GPRINT:freepct:AVERAGE:Ave free\\:   %3.2lf%%",
                           "GPRINT:freepct:LAST:Current free\\:  %3.2lf%%\\j" )
    else:
        print DB_DIR + "/" + DB + ".rrd not found! Skipping graph for", DB

    return

def upsgraph():

    DB = 'ups'
    if os.path.isfile( DB_DIR + "/" + DB + ".rrd" ):
        for p in periods:
            rrdtool.graph( PNG_DIR + '/' + DB + '-' + p + '.png',
                           '-s -1' + p, "-aPNG",
                           "--alt-y-grid", "-w 600", "-h 100", "-l 0", "-E",
                           "--color", "SHADEA#EAE9EE",
                           "--color", "SHADEB#EAE9EE",
                           "--color", "BACK#EAE9EE",
                           "-t", model + '- ' + p + ' | Current status: ' + status,
                           'DEF:load=' + DB_DIR + '/' + DB + '.rrd:load:AVERAGE',
                           'DEF:line=' + DB_DIR + '/' + DB + '.rrd:line:AVERAGE',
                           'DEF:charge=' + DB_DIR + '/' + DB + '.rrd:charge:AVERAGE',
                           'DEF:time=' + DB_DIR + '/' + DB + '.rrd:time:AVERAGE',
                           "LINE2:load#0000FF:Load\\:",
                           "GPRINT:load:LAST: %.1lf %%",
                           "LINE2:line#00FF00:Input\\:",
                           "GPRINT:line:LAST: %.1lf V",
                           "LINE2:charge#FF0000:Charge\\:",
                           "GPRINT:charge:LAST:%.1lf %%",
                           "LINE2:time#CC3232:Battery Time\\:",
                           "GPRINT:time:LAST:%.1lf min\\j" )

    else:
        print DB_DIR + "/" + DB + ".rrd not found! Skipping graph for", DB

    return

def createdb():

    for opt in cli_opts:
        cmd = opt + 'db()'
        cmd = cmd.lstrip('--')
        exec cmd

    return

def updatedb():

    for opt in cli_opts:
        cmd = opt + 'up()'
        cmd = cmd.lstrip('--')
        if cmd == 'usageup()':
            partition_map = usagedb()
            cmd = 'usageup(partition_map)'
        elif cmd == 'hddtempup()':
            disk_map = hddtempdb()
            cmd = 'hddtempup(disk_map)'
        elif cmd == 'fansup()':
            total_fans = fansdb()
            cmd = 'fansup(total_fans)'
        elif cmd == 'activityup()':
            drive_activity = activitydb()
            cmd = 'activityup(drive_activity)'
        exec cmd

    return

def graph():

    for opt in cli_opts:
        cmd = opt + 'graph()'
        cmd = cmd.lstrip('--')
        if cmd == 'usagegraph()':
            partition_map = usagedb()
            cmd = 'usagegraph(partition_map)'
        elif cmd == 'hddtempgraph()':
            disk_map = hddtempdb()
            cmd = 'hddtempgraph(disk_map)'
        elif cmd == 'fansgraph()':
            total_fans = fansdb()
            cmd = 'fansgraph(total_fans)'
        elif cmd == 'activitygraph()':
            drive_activity = activitydb()
            cmd = 'activitygraph(drive_activity)'
        exec cmd

    return

commandline()
createdb()
updatedb()
graph()
